﻿// Painting Controls With Fade Animations Using the Buffered Paint API
// 2011/10/23 - Bradley Smith

using System;
using System.Drawing;
using System.Windows.Forms;

namespace BufferedPainting {

    /// <summary>
    /// Example control that uses the BufferedPainter class. 
    /// Inherits from UserControl to provide focus state.
    /// </summary>
    public class ExampleControl : UserControl {

        public enum MyVisualStates {
            Normal,
            Hot,
            Pushed
        }

        BufferedPainter<MyVisualStates> painter;

        public ExampleControl() {
            painter = new BufferedPainter<MyVisualStates>(this);
            painter.PaintVisualState += new EventHandler<BufferedPaintEventArgs<MyVisualStates>>(painter_PaintState);
            painter.DefaultDuration = 125;

            painter.AddTransition(MyVisualStates.Hot, MyVisualStates.Pushed, 75);
            painter.AddTransition(MyVisualStates.Hot, MyVisualStates.Normal, 250);
            painter.AddTransition(MyVisualStates.Pushed, MyVisualStates.Normal, 250);

            painter.AddTrigger(VisualStateTriggerTypes.Hot, MyVisualStates.Hot);
            painter.AddTrigger(VisualStateTriggerTypes.Pushed, MyVisualStates.Pushed);
        }

        void painter_PaintState(object sender, BufferedPaintEventArgs<MyVisualStates> e) {
            Color color;            
            
            switch (e.State) {
                case MyVisualStates.Hot:
                    color = Color.Blue;
                    break;
                case MyVisualStates.Pushed:
                    color = Color.Orange;
                    break;
                default:
                    color = Color.Gray;
                    break;
            }

            e.Graphics.Clear(color);

            if (Focused) {
                Rectangle focus = ClientRectangle;
                focus.Inflate(-2, -2);
                ControlPaint.DrawFocusRectangle(e.Graphics, focus);
            }
        }

        protected override void OnPaintBackground(PaintEventArgs pevent) {
            // don't paint the control's background
        }

        protected override void OnLostFocus(EventArgs e) {
            base.OnLostFocus(e);

            // UserControl doesn't repaint when focus is lost
            Invalidate();
        }

        protected override void OnGotFocus(EventArgs e) {
            base.OnGotFocus(e);

            // UserControl doesn't repaint when focus is gained
            Invalidate();
        }
    }

}